(*
Copyright 2011 Christian Sternagel, René Thiemann

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory Check_CRP
imports Check_TP 
begin

subsection {*Proving confluence of TRSs*}

datatype ('f, 'l, 'v) cr_proof =
  SN_WCR "(('f, 'l) lab, 'v) crit_pair_joins option" "('f, 'l, 'v) trs_proof" 

fun
  check_cr_proof ::
    "bool \<Rightarrow> shows \<Rightarrow>
     ('tp, ('f::{show,linorder}, label_type) lab, string) tp_ops \<Rightarrow>
     ('dpp, ('f, label_type) lab, string) dpp_ops \<Rightarrow> (('f,label_type)lab,string) rules \<Rightarrow>
     ('f, label_type, string) cr_proof \<Rightarrow>
     shows check" 
where
  "check_cr_proof a i I J R (SN_WCR joins prf) = debug (i []) ''SN_WCR'' (do {
         let tp = tp_ops.mk I [] R [];
         check_trs_proof a (i +@+ shows_string ''.1'') I J tp prf
           <+? (\<lambda>s. i +@+ '': error below strong normalization + wcr'' +#+ shows_nl
             +@+ indent s);
         check_critical_pairs R (critical_pairs_impl R R) joins
           <+? (\<lambda>s. i +@+ shows ''error when proving local confluence of '' +@+ shows_tp I tp +@+ shows_nl +@+ indent s)
   })" 

fun cr_assms :: "bool \<Rightarrow> ('f, 'l, 'v)cr_proof \<Rightarrow> ('f, 'l, 'v) prob list" where
  "cr_assms a (SN_WCR _ p) = assms a p"

lemma check_cr_proof_with_assms_sound: 
  assumes I: "tp_spec I" and J: "dpp_spec J"
    and fin: "\<forall>p\<in>set (cr_assms a prf). holds p"
    and ok: "isOK (check_cr_proof a i I J R prf)"
  shows "CR (rstep (set R))"
proof -
  interpret tp_spec I by fact
  from ok fin show ?thesis
  proof (induct "prf" arbitrary: i)
    case (SN_WCR joins prof i)
    from SN_WCR(1)
    have cp: "isOK(check_critical_pairs R (critical_pairs_impl R R) joins)"
      and ok: "isOK(check_trs_proof a (i +@+ shows_string ''.1'') I J (mk [] R []) prof)" by (auto simp: Let_def)
    from SN_WCR(2) have a: "\<forall> a \<in> set (assms a prof). holds a" by auto
    from check_critical_pairs[OF cp] have WCR: "WCR (rstep (set R))" .
    from check_trs_proof_with_assms_sound[OF I J a ok] have SN: "SN (rstep (set R))" by simp
    from Newman[OF SN WCR] show ?case .
  qed
qed

lemma cr_assms_False[simp]: "cr_assms False prf = []"
  by (induct False "prf" rule: cr_assms.induct) simp_all

lemma check_cr_proof_sound: 
  assumes I: "tp_spec I" and J: "dpp_spec J"
  and ok: "isOK (check_cr_proof False i I J R prf)"
  shows "CR (rstep (set R))"
  by (rule check_cr_proof_with_assms_sound[OF I J _ ok], simp)

end