(*
Copyright 2011 Christian Sternagel, René Thiemann

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory Check_DPP
imports Check_Common
begin

subsection {*Proving finiteness in the QDP Framework*}

datatype ('f, 'l, 'v) dp_proof =
  PisEmpty
| SubtermCriterionProc "('f, 'l) lab projL" "('f, 'l, 'v) rseqL"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof"
| RedPairProc "('f,'l)lab root_redtriple_impl + ('f, 'l) lab redtriple_impl" "('f, 'l, 'v) trsLL"  "('f, 'l, 'v) dp_proof"
| RedPairUrProc "('f, 'l) lab root_redtriple_impl + ('f, 'l) lab redtriple_impl"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof" 
| UsableRulesProc "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof" 
| DepGraphProc "(('f, 'l, 'v) dp_proof option \<times> ('f, 'l, 'v) trsLL) list"
| MonoRedPairProc "('f, 'l) lab redtriple_impl"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof" 
| MonoRedPairUrProc "('f, 'l) lab redtriple_impl"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof"
| SizeChangeSubterm "((('f, 'l) lab, 'v) rule \<times> ((nat \<times> nat) list \<times> (nat \<times> nat) list)) list"
| SizeChangeRedPair "('f, 'l) lab redtriple_impl" "('f, 'l, 'v) trsLL option"
    "((('f, 'l) lab, 'v) rule \<times> ((nat \<times> nat) list \<times> (nat \<times> nat) list)) list"
| UncurryProc "nat option" "(('f, 'l) lab, 'v) uncurry_info"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof"
| FccProc "('f, 'l) lab" "(('f, 'l) lab, 'v) ctxt list"
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" "('f, 'l, 'v) dp_proof"
| SplitProc
    "('f, 'l, 'v) trsLL" "('f, 'l, 'v) trsLL" 
    "('f, 'l, 'v) dp_proof" "('f, 'l, 'v) dp_proof"
| SemlabProc
    "(('f, 'l) lab, 'v) sl_variant" "('f, 'l, 'v) trsLL"
    "(('f, 'l) lab, 'v) term list" "('f, 'l, 'v) trsLL"
    "('f, 'l, 'v) dp_proof"
| SwitchInnermostProc "(('f,'l)lab,'v)crit_pair_joins option" "('f, 'l, 'v)dp_proof"
| RewritingProc   "('f, 'l, 'v) trsLL" "('f,'l,'v)ruleLL" "('f,'l,'v)ruleLL" "('f,'l,'v)ruleLL" pos "('f, 'l, 'v) dp_proof"
| InstantiationProc   "('f,'l,'v)ruleLL" "('f,'l,'v)trsLL" "('f, 'l, 'v) dp_proof"
| ForwardInstantiationProc   "('f,'l,'v)ruleLL" "('f,'l,'v)trsLL" "('f,'l,'v)trsLL option" "('f, 'l, 'v) dp_proof"
| NarrowingProc   "('f,'l,'v)ruleLL" pos "('f,'l,'v)trsLL" "('f, 'l, 'v) dp_proof"
| AssumeFinite "('f, 'l, 'v) dppLL"

definition missing :: "string \<Rightarrow> shows \<Rightarrow> shows" where
  "missing s x \<equiv> ''the '' +#+ s +#+ '' '' +#+ x +@+ shows_string '' is missing''"

definition toomuch :: "string \<Rightarrow> shows \<Rightarrow> shows" where
  "toomuch s x \<equiv> ''superfluous '' +#+ s +#+ '' '' +#+ x +@+ x"

fun
  check_dpp_subsumes ::
    "('d, ('f::show, 'l::show) lab, 'v::show) dpp_ops
    \<Rightarrow> ('f, 'l, 'v) dppLL \<Rightarrow> 'd \<Rightarrow> shows check"
where
  "check_dpp_subsumes I (p', pw', q', r', rw') d = (do {
    let p  = dpp_ops.P I d;
    let pw = dpp_ops.Pw I d;
    let q  = dpp_ops.Q I d;
    let r  = dpp_ops.R I d;
    let rw = dpp_ops.Rw I d;
    let pb' = p' @ pw';
    let rb  = r  @ rw;
    let rb' = r' @ rw';
    check_subseteq p p'   <+? (\<lambda>r. toomuch ''pair'' (shows_rule r));
    check_subseteq pw pb' <+? (\<lambda>r. toomuch ''weak pair'' (shows_rule r));
    check_NF_terms_eq q q'   <+? (\<lambda>r. shows ''NF(Q) differs due to term '' +@+ shows_term r);
    check_subseteq r r'   <+? (\<lambda>r. toomuch ''strict rule'' (shows_rule r));
    check_subseteq rb rb' <+? (\<lambda>r. toomuch ''strict/weak rule'' (shows_rule r));
    check_subseteq rb' rb <+? (\<lambda>r. missing ''strict/weak rule'' (shows_rule r));
    return ()
  }) <+? (\<lambda>s. ''finiteness of the problem'' +#+ shows_nl +@+ shows_dpp I d +@+ shows_nl
           +@+ ''may not be concluded from assuming finiteness of the problem'' +#+ shows_nl
           +@+ shows_dpp I (dpp_ops.mk I p' pw' q' r' rw') +@+ shows_nl
           +@+ s +@+ shows_nl)"

lemma check_dpp_subsumesE[elim]:
  assumes 1: "dpp_spec I"
    and 2: "isOK (check_dpp_subsumes I (p, pw, q, r, rw) d)"
    and 3: "set (dpp_ops.P I d) \<subseteq> set p
    \<Longrightarrow> set (dpp_ops.Pw I d) \<subseteq> set p \<union> set pw
    \<Longrightarrow> NF_terms (set (dpp_ops.Q I d)) = NF_terms (set q)
    \<Longrightarrow> set (dpp_ops.R I d) \<subseteq> set r
    \<Longrightarrow> set (dpp_ops.R I d) \<union> set (dpp_ops.Rw I d) = set r \<union> set rw
    \<Longrightarrow> P"
  shows "P"
  by (rule 3, insert 1 2, auto simp: Let_def)

fun finite_dppL :: "('f, 'l, 'v) dppLL \<Rightarrow> bool" where
  "finite_dppL (p, pw, q, r, rw) = finite_dpp (set p, set pw, set q, set r, set rw)"

lemma check_dpp_subsumes_sound:
  assumes I: "dpp_spec I"
    and ok: "isOK (check_dpp_subsumes I dpp' dpp)"
    and fin: "finite_dppL dpp'"
  shows "finite_dpp (dpp_ops.dpp I dpp)"
proof -
  obtain p pw q r rw where dpp': "dpp' = (p, pw, q, r, rw)" by (cases dpp') force+
  let ?P   = "set (dpp_ops.P I dpp)"
  let ?Pw  = "set (dpp_ops.Pw I dpp)"
  let ?Q   = "set (dpp_ops.Q I dpp)"
  let ?R   = "set (dpp_ops.R I dpp)"
  let ?Rw  = "set (dpp_ops.Rw I dpp)"
  let ?P'  = "set p"
  let ?Pw' = "set pw"
  let ?Q'  = "set q"
  let ?R'  = "set r"
  let ?Rw' = "set rw"
  from I and ok
    have "?P \<subseteq> ?P'" and "?Pw \<subseteq> ?P' \<union> ?Pw'" and q: "NF_terms ?Q = NF_terms ?Q'" and "?R \<subseteq> ?R'"
    and "?R \<union> ?Rw = ?R' \<union> ?Rw'"
    unfolding dpp' by blast+
  note 1 = this
  show ?thesis unfolding dpp_spec.dpp_sound[OF I]
    by (rule finite_dpp_mono[OF fin[unfolded dpp' finite_dppL.simps]], insert 1, auto)
qed

(*The first argument is a flag that specifies whether partial proofs (i.e, containing
assumptions) are accepted or not.*)
function
  check_dp_proof ::
    "bool \<Rightarrow> shows \<Rightarrow> ('dpp, ('f::{show,linorder},label_type) lab, string) dpp_ops \<Rightarrow> 'dpp \<Rightarrow>
    ('f, label_type, string) dp_proof \<Rightarrow>
    shows check"
where 
  "check_dp_proof assms i I dpp PisEmpty = debug (i []) ''P is empty''
     (if dpp_ops.P I dpp = [] \<and> (dpp_ops.Pw I dpp = [] \<or> dpp_ops.R I dpp = []) 
          then return () 
          else error (i +@+ '': P is not empty in the following DP-problem '' +#+
         shows_nl +@+ shows_dpp I dpp))"
| "check_dp_proof assms i I dpp (SubtermCriterionProc p rseq rP prf) =
    debug (i []) ''SubtermCriterionProc'' (do {
      let P  = dpp_ops.pairs I dpp;
      let Pr = list_diff P rP;
      dpp' \<leftarrow> subterm_criterion_proc I (create_proj p) (create_rseq_map rseq) Pr dpp
        <+? (\<lambda>s. i +@+ '': error when applying the subterm criterion to the DP problem''
          +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
          ''and trying to remove the pairs '' +#+ shows_nl +@+ shows_rules Pr
          +@+ shows_nl +@+ s);
      check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
        <+? (\<lambda>s. i +@+ '': error below the subterm criterion'' +#+ shows_nl
          +@+ indent s)
    })"
| "check_dp_proof assms i I dpp (RedPairProc redp rP prf) = debug (i []) ''RedPairProc'' (do {
         let P = dpp_ops.pairs I dpp;
         let Pr = list_diff P rP;
         let Proc = (case redp of Inr rp \<Rightarrow> generic_ur_af_redtriple_proc I (get_redtriple rp) None
                        | Inl rrp \<Rightarrow> generic_ur_af_root_redtriple_proc I (get_root_redtriple rrp) None);
         dpp' \<leftarrow> Proc Pr dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the reduction pair processor to remove from the DP problem''
             +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
             '' the pairs '' +#+ shows_nl +@+ shows_rules Pr +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the reduction pair processor'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (UsableRulesProc U prf) = debug (i []) ''UsableRulesProc'' (do {
         dpp' \<leftarrow> usable_rules_proc I U dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the usable rules processor to restrict the DP problem''
             +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
             '' to the usable rules '' +#+ shows_nl +@+ shows_rules U +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the usable rules processor'' +#+ shows_nl
             +@+ indent s)
   })"
|  "check_dp_proof assms i I dpp (MonoRedPairProc redp rP rR prf) = debug (i []) ''MonoRedPairProc'' (do {
         let P = dpp_ops.pairs I dpp;
         let Pr = list_diff P rP;
         let R = dpp_ops.rules I dpp;
         let Rr = list_diff R rR;
         dpp' \<leftarrow> mono_redpair_proc I (get_redtriple redp) Pr Rr dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the mono reduction pair processor to remove from the DP problem''
             +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
             '' the pairs '' +#+ shows_nl +@+ shows_rules Pr +@+ shows_nl +@+ 
             '' and the rules '' +#+ shows_nl +@+ shows_rules Rr +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the mono reduction pair processor'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (DepGraphProc edpts) =
     debug (i []) ''DepGraphProc'' (do {
        pdpps \<leftarrow> dep_graph_proc I dpp edpts
         <+? (\<lambda>s. i +@+ '': error while trying to perform SCC-decomposition  on '' +#+
           shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ s);
        check_allm_index (\<lambda> (prof,dpp') j. check_dp_proof assms (i +@+ ''.'' +#+ shows (Suc j)) I dpp' prof) pdpps
          <+? (\<lambda>s. i +@+ '': error below the dependency graph processor'' +#+ shows_nl
            +@+ indent s)
  })"
| "check_dp_proof assms i I dpp (RedPairUrProc redp rP ur prf) = debug (i []) ''RedPairUrProc'' (do {
         let P = dpp_ops.pairs I dpp;
         let Pr = list_diff P rP;
         let Proc = (case redp of Inr rp \<Rightarrow> generic_ur_af_redtriple_proc I (get_redtriple rp) (Some ur) 
                        | Inl rrp \<Rightarrow> generic_ur_af_root_redtriple_proc I (get_root_redtriple rrp) (Some ur));
         dpp' \<leftarrow> Proc Pr dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the reduction pair processor with usable rules to remove from the DP problem''
             +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
             '' the pairs '' +#+ shows_nl +@+ shows_rules Pr +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the reduction pair processor'' +#+ shows_nl
             +@+ indent s)
   })" 
|  "check_dp_proof assms i I dpp (MonoRedPairUrProc redp rP rR ur prf) = debug (i []) ''MonoRedPairUrProc'' (do {
         let P = dpp_ops.pairs I dpp;
         let Pr = list_diff P rP;
         let R = dpp_ops.rules I dpp;
         let Rr = list_diff R rR;
         dpp' \<leftarrow> mono_ur_redpair_proc I (get_redtriple redp) Pr Rr ur dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the mono reduction pair processor with usable rules to remove from the DP problem''
             +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+
             '' the pairs '' +#+ shows_nl +@+ shows_rules Pr +@+ shows_nl +@+ 
             '' and the rules '' +#+ shows_nl +@+ shows_rules Rr +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the mono reduction pair processor with usable rules'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (UncurryProc mode u_info P' R' prf) = 
    debug (i []) ''UncurryProc'' (do {
      dpp' \<leftarrow> uncurry_proc_both I mode u_info P' R' dpp
        <+? (\<lambda>s. i +@+ '': error when applying the uncurrying processor on the DP problem''
          +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ s);
      check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
        <+? (\<lambda>s. i +@+ '': error below the uncurrying processor'' +#+ shows_nl
          +@+ indent s)
    })"
| "check_dp_proof assms i I dpp (SizeChangeSubterm graphs) =
     debug (i []) ''SizeChangeSubterm'' (sct_subterm_proc I graphs dpp)"
| "check_dp_proof assms i I dpp (SizeChangeRedPair redp U_opt graphs) =
     debug (i []) ''SizeChangeRedPair'' (sct_ur_af_proc I (get_redtriple redp) graphs U_opt dpp)"
| "check_dp_proof assms i I dpp (FccProc f fcs P' R' prf) = debug (i []) ''FccProc'' (do {
    dpp' \<leftarrow> fcc_proc I f fcs P' [] R' dpp
      <+? (\<lambda>s. i +@+
        '': error when applying the flat context closure processor on the DP problem'' +#+
        shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ s);
    check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
      <+? (\<lambda>s. i +@+ '': error below the flat context closure processor'' +#+ shows_nl
        +@+ indent s)
  })"
| "check_dp_proof assms i I dpp (SplitProc Prem Rrem prf1 prf2) = debug (i []) ''SplitProc'' (do {
    let (dpp1,dpp2) = split_proc I dpp Prem Rrem;
    check_dp_proof assms (i +@+ shows_string ''.1'') I dpp1 prf1
      <+? (\<lambda>s. i +@+ '': error below the split processor'' +#+ shows_nl
        +@+ indent s);
    check_dp_proof assms (i +@+ shows_string ''.2'') I dpp2 prf2
      <+? (\<lambda>s. i +@+ '': error below the split processor'' +#+ shows_nl
        +@+ indent s)
  })"
| "check_dp_proof assms i I dpp (SemlabProc sli lP lQ lR prf) = 
    debug (i []) ''SemlabProc'' (do {
      dpp' \<leftarrow> semlab_fin_proc I sli lP lQ lR dpp
        <+? (\<lambda>s. i +@+ '': error when applying the semlab processor on the DP problem''
          +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ s);
      check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
        <+? (\<lambda>s. i +@+ '': error below the semlab processor'' +#+ shows_nl
          +@+ indent s)
    })"
| "check_dp_proof assms i I dpp (SwitchInnermostProc joins prf) = 
    debug (i []) ''SwitchInnermostProc'' (do {
      dpp' \<leftarrow> switch_innermost_proc I joins dpp
        <+? (\<lambda>s. i +@+ '': error when applying the processor to switch to innermost on the DP problem''
          +#+ shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ s);
      check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
        <+? (\<lambda>s. i +@+ '': error below the switch to innermost processor'' +#+ shows_nl
          +@+ indent s)
    })"
| "check_dp_proof assms i I dpp (AssumeFinite dpp') =
    debug (i []) ''Finiteness Assumption'' (
      if assms
        then check_dpp_subsumes I dpp' dpp
          <+? (\<lambda>s. i +@+ '': error in finiteness assumption'' +#+ shows_nl +@+ s +@+ shows_nl)
        else error (i +@+ '': the proof contains a local finiteness assumption'' +#+ shows_nl)
    )"
| "check_dp_proof assms i I dpp (RewritingProc U st st' lr p prf) = debug (i []) ''RewritingProc'' (do {
         dpp' \<leftarrow> rewriting_proc I U st st' lr p dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the rewriting processor to rewrite the pair ''
             +#+ shows_nl +@+ shows_rule st +@+ shows_nl +@+
             '' to the pair '' +#+ shows_nl +@+ shows_rule st' +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the rewriting processor'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (NarrowingProc st p sts prf) = debug (i []) ''NarrowingProc'' (do {
         dpp' \<leftarrow> narrowing_proc I st p sts dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the processor processor to narrow the pair ''
             +#+ shows_nl +@+ shows_rule st +@+ shows_nl +@+
             '' to the pairs '' +#+ shows_nl +@+ shows_trs sts +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the narrowing processor'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (InstantiationProc st sts prf) = debug (i []) ''InstantiationProc'' (do {
         dpp' \<leftarrow> instantiation_proc I st sts dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the instantiation processor to instantiate the pair ''
             +#+ shows_nl +@+ shows_rule st +@+ shows_nl +@+
             '' to the pairs '' +#+ shows_nl +@+ shows_trs sts +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the instantiation processor'' +#+ shows_nl
             +@+ indent s)
   })" 
| "check_dp_proof assms i I dpp (ForwardInstantiationProc st sts u_opt prf) = debug (i []) ''ForwardInstantiationProc'' (do {
         dpp' \<leftarrow> forward_instantiation_proc I st sts u_opt dpp 
           <+? (\<lambda>s. i +@+ '': error when applying the forward_instantiation processor to instantiate the pair ''
             +#+ shows_nl +@+ shows_rule st +@+ shows_nl +@+
             '' to the pairs '' +#+ shows_nl +@+ shows_trs sts +@+ shows_nl +@+ s);
         check_dp_proof assms (i +@+ shows_string ''.1'') I dpp' prf
           <+? (\<lambda>s. i +@+ '': error below the forward instantiation processor'' +#+ shows_nl
             +@+ indent s)
   })" 
  by pat_completeness auto

termination
proof (rule, unfold update_error_return)
  fix a i I dpp 
    and edpts::"(('f::{show,linorder},'l::{show,linorder}, string) dp_proof option
      \<times> ('f, 'l, string ) trsLL) list" 
    and pdpps pdpp i' prof dpp' j
  assume proc: "dep_graph_proc I dpp edpts = Inr pdpps"
    and mem: "pdpp \<in> set pdpps" and pdpp: "(prof, dpp') = pdpp"
  from proc mem[unfolded pdpp[symmetric]]
    have "Some prof \<in> set (map fst edpts)" unfolding dep_graph_proc_def by force
  then obtain P where mem: "(Some prof, P) \<in> set edpts" by auto
  let ?m = "prod_size (option_size size)
              (list_size
                (prod_size (Term.term.term_size (lab_size (\<lambda>x. 0) (\<lambda>x. 0)) (\<lambda>x. 0))
                  (Term.term.term_size (lab_size (\<lambda>x. 0) (\<lambda>x. 0)) (\<lambda>x. 0))))"
  have "size prof < ?m (Some prof, P)" by auto
  also have "... \<le> list_size ?m edpts" using mem by (induct edpts, auto)
  finally have "size prof < size (DepGraphProc edpts)" by simp
  thus "((a, i +@+ ''.'' +#+ shows (Suc j), I, dpp', prof), a, i, I, dpp, DepGraphProc edpts)
    \<in> measures [\<lambda>(a, i, I, dpp, prof). size prof]" by auto
qed auto

lemma isOK_check_dp_proof_AssumeFinite_False[simp]:
  "isOK (check_dp_proof False i I dpp (AssumeFinite dpp')) = False" by simp


fun dp_assms :: "bool \<Rightarrow> ('f, 'l, 'v) dp_proof \<Rightarrow> ('f, 'l, 'v) dppLL list" where
  "dp_assms a PisEmpty = []"
| "dp_assms a (SubtermCriterionProc _ _ _ p) = dp_assms a p"
| "dp_assms a (RedPairProc _ _ p) = dp_assms a p"
| "dp_assms a (RedPairUrProc _ _ _ p) = dp_assms a p"
| "dp_assms a (UsableRulesProc _ p) = dp_assms a p"
| "dp_assms a (DepGraphProc ps) =
    concat (concat (map (map (dp_assms a) \<circ> option_to_list \<circ> fst) ps))"
| "dp_assms a (MonoRedPairProc _ _ _ p) = dp_assms a p"
| "dp_assms a (MonoRedPairUrProc _ _ _ _ p) = dp_assms a p"
| "dp_assms a (SizeChangeSubterm _) = []"
| "dp_assms a (SizeChangeRedPair _ _ _) = []"
| "dp_assms a (UncurryProc _ _ _ _ p) = dp_assms a p"
| "dp_assms a (FccProc _ _ _ _ p) = dp_assms a p"
| "dp_assms a (SplitProc _ _ p q) = dp_assms a p @ dp_assms a q"
| "dp_assms a (SemlabProc _ _ _ _ p) = dp_assms a p"
| "dp_assms a (SwitchInnermostProc _ p) = dp_assms a p"
| "dp_assms a (RewritingProc _ _ _ _ _ p) = dp_assms a p"
| "dp_assms a (NarrowingProc _ _ _ p) = dp_assms a p"
| "dp_assms a (InstantiationProc _ _ p) = dp_assms a p"
| "dp_assms a (ForwardInstantiationProc _ _ _ p) = dp_assms a p"
| "dp_assms a (AssumeFinite d) = (if a then [d] else [])"


lemma dp_assms_False[simp]: "dp_assms False prf = []"
  by (induct False "prf" rule: dp_assms.induct) simp_all

lemma check_dp_proof_with_dp_assms:
  assumes I: "dpp_spec I"
    and fin: "\<forall>p\<in>set (dp_assms a prf). finite_dppL p"
    and ok: "isOK (check_dp_proof a i I dpp prf)"
  shows "finite_dpp (dpp_ops.dpp I dpp)"
using ok I fin
proof (induct a i I dpp "prf" rule: check_dp_proof.induct)
  case (1 a i I dpp)
  interpret dpp_spec I by fact
  from 1(1) show ?case by (auto simp:
    QDP_Framework.finite_dpp_def dpp_spec_sound QDP_Framework.ichain.simps)
next
  case (2 a i I dpp p rseq rP prof)
  interpret dpp_spec I by fact
  let ?Pr = "list_diff (dpp_ops.pairs I dpp) rP"
  let ?i = "i +@+ shows_string ''.1''"
  from 2(2)
    obtain dpp' where proc: "subterm_criterion_proc I (create_proj p) (create_rseq_map rseq) ?Pr dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 2 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case
    by (rule sound_proc_impl[OF subterm_criterion_proc,
      OF proc 2(1)[OF HOL.refl HOL.refl _ rec 2(3) fin]])
      (unfold update_error_return, rule proc)
next
  case (3 a i I dpp rpc rP prof)
  note IH = 3
  interpret dpp_spec I by fact
  let ?Pr = "list_diff (dpp_ops.pairs I dpp) rP"
  let ?i = "i +@+ shows_string ''.1''"
  from IH have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case 
  proof (cases rpc)
    case (Inr rp)
    with IH(2)
    obtain dpp' where proc: "generic_ur_af_redtriple_proc I (get_redtriple rp) None ?Pr dpp = return dpp'"
      and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
      by (auto simp: Let_def)
    from IH(1)[OF HOL.refl HOL.refl HOL.refl _ rec IH(3) fin, unfolded Inr update_error_return sum.simps, OF proc]
    have fin: "finite_dpp (dpp_ops.dpp I dpp')" .
    show ?thesis      
      by (rule sound_proc_impl[OF generic_ur_af_redtriple_proc[OF IH(3) get_redtriple] _ fin], 
        rule proc)
  next
    case (Inl rrp)
    with IH(2)
    obtain dpp' where proc: "generic_ur_af_root_redtriple_proc I (get_root_redtriple rrp) None ?Pr dpp = return dpp'"
      and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
      by (auto simp: Let_def)
    from IH(1)[OF HOL.refl HOL.refl HOL.refl _ rec IH(3) fin, unfolded Inl update_error_return sum.simps, OF proc] 
    have fin: "finite_dpp (dpp_ops.dpp I dpp')" .
    show ?thesis
      by (rule sound_proc_impl[OF generic_ur_af_root_redtriple_proc[OF IH(3) get_root_redtriple] _ fin], rule proc)
  qed
next
  case (4 a i I dpp ur prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 4(2) obtain dpp'
    where proc: "usable_rules_proc I ur dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 4 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case
    by (rule sound_proc_impl[OF usable_rules_proc, OF _ proc 4(1)[OF _ rec _ fin]], insert proc 4(3), auto)  
next
  case (5 a i I dpp rp rP rR prof)
  interpret dpp_spec I by fact
  let ?Pr = "list_diff (dpp_ops.pairs I dpp) rP"
  let ?Rr = "list_diff (dpp_ops.rules I dpp) rR"
  let ?i = "i +@+ shows_string ''.1''"
  from 5(2)
    obtain dpp' where proc: "mono_redpair_proc I (get_redtriple rp) ?Pr ?Rr dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 5 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case      
    by (rule sound_proc_impl[OF mono_redpair_proc[OF get_redtriple],
      OF proc 5(1)[OF HOL.refl HOL.refl HOL.refl HOL.refl _ rec 5(3) fin]],
      unfold update_error_return, rule proc)
next
  case (6 a i I dpp edpts)
  interpret dpp_spec I by fact
  from 6(2) obtain pdpps where proc: "dep_graph_proc I dpp edpts = return pdpps" 
    and rec: "\<And>j. j < length pdpps \<Longrightarrow>
      isOK ((\<lambda>(prof, dpp') j.
        check_dp_proof a (i +@+ ''.'' +#+ shows (Suc j)) I dpp' prof) (pdpps ! j) j)"
    by auto
  show ?case
  proof (rule dep_graph_proc[OF 6(3) proc])
    fix p dpp'
    assume mem: "(p, dpp') \<in> set pdpps"
    from this[unfolded set_conv_nth]
      obtain j where j: "j < length pdpps" and pair: "pdpps ! j = (p, dpp')" by auto
    from rec[OF j] have rec: "isOK (check_dp_proof a (i +@+ ''.'' +#+ shows (Suc j)) I dpp' p)"
      unfolding pair by simp
    from proc[unfolded dep_graph_proc_def, simplified]
      have pdpps: "pdpps =
      map (\<lambda>aP. (the (fst aP), dpp_ops.intersect_pairs I dpp (snd aP)))
          [aP\<leftarrow>edpts . \<exists>y. fst aP = Some y]" by simp
    from mem and 6(4) and proc
      have fin: "\<forall>a\<in>set (dp_assms a p). finite_dppL a" by (simp add: dep_graph_proc_def) force
    show "finite_dpp (dpp_ops.dpp I dpp')" 
      by (rule 6(1)[unfolded update_error_return, OF proc mem HOL.refl rec 6(3) fin])
  qed
next
  case (7 a i I dpp rpc rP ur prof)
  note IH = 7
  interpret dpp_spec I by fact
  let ?Pr = "list_diff (dpp_ops.pairs I dpp) rP"
  let ?i = "i +@+ shows_string ''.1''"
  from IH have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case 
  proof (cases rpc)
    case (Inr rp)
    with IH(2) obtain dpp'
      where proc: "generic_ur_af_redtriple_proc I (get_redtriple rp) (Some ur) ?Pr dpp = return dpp'"
      and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
      by (auto simp: Let_def)
    show ?thesis      
      by (rule sound_proc_impl[OF generic_ur_af_redtriple_proc[OF IH(3) get_redtriple],
        OF proc IH(1)[OF HOL.refl HOL.refl HOL.refl _ rec IH(3) fin]],
        unfold Inr update_error_return sum.simps, rule proc)
  next
    case (Inl rrp)
    with IH(2) obtain dpp'
      where proc: "generic_ur_af_root_redtriple_proc I (get_root_redtriple rrp) (Some ur) ?Pr dpp = return dpp'"
      and rec: "isOK (check_dp_proof a ?i I dpp' prof)" 
      by (auto simp: Let_def)
    show ?thesis      
      by (rule sound_proc_impl[OF generic_ur_af_root_redtriple_proc[OF IH(3) get_root_redtriple],
        OF proc IH(1)[OF HOL.refl HOL.refl HOL.refl _ rec IH(3) fin]],
        unfold Inl update_error_return sum.simps, rule proc)
  qed
next
  case (8 a i I dpp rp rP rR ur prof)
  interpret dpp_spec I by fact
  let ?Pr = "list_diff (dpp_ops.pairs I dpp) rP"
  let ?Rr = "list_diff (dpp_ops.rules I dpp) rR"
  let ?i = "i +@+ shows_string ''.1''"
  from 8(2) obtain dpp'
    where proc: "mono_ur_redpair_proc I (get_redtriple rp) ?Pr ?Rr ur dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 8 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case
    by (rule sound_proc_impl[OF mono_ur_redpair_proc[OF get_redtriple],
      OF proc 8(1)[OF HOL.refl HOL.refl HOL.refl HOL.refl _ rec 8(3) fin]],
      unfold update_error_return, rule proc)
next
  case (9 a i I dpp mode u_info P' R' prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 9(2) obtain
    dpp' where proc: "uncurry_proc_both I mode u_info P' R'  dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 9 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case      
    by (rule sound_proc_impl[OF uncurry_proc_both[OF 9(3)], OF proc 9(1)],
      unfold update_error_return, rule proc, rule rec, rule 9(3), rule fin)
next
  case (10 a i I dpp graphs)
  interpret dpp_spec I by fact
  from 10(1) have proc: "isOK (sct_subterm_proc I graphs dpp)" by auto
  show ?case
    by (rule sct_subterm_proc_sound[OF proc])
next
  case (11 a i I dpp rp u_opt graphs)
  note IH = 11
  interpret dpp_spec I by fact
  from IH(1) have proc: "isOK (sct_ur_af_proc I (get_redtriple rp) graphs u_opt dpp)" by auto
  show ?case
    by (rule sct_ur_af_proc[OF IH(2) get_redtriple, OF proc])
next
  case (12 a i I dpp f fcs P' R' prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 12(2) obtain dpp'
    where proc: "fcc_proc I f fcs P' [] R' dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)" by auto
  from 12 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  from 12(1)[OF _ rec 12(3) fin, simplified, OF proc] have "finite_dpp (dpp_ops.dpp I dpp')"
    by (simp add: dpp_spec_sound)
  from sound_proc_impl[OF fcc_proc, OF proc this] show ?case .
next
  case (13 a i I dpp P' R' prof1 prof2)
  interpret dpp_spec I by fact
  let ?i = "\<lambda>s. i +@+ shows_string s"
  let ?i1 = "?i ''.1''"
  let ?i2 = "?i ''.2''"
  obtain dpp1 dpp2 where s: "split_proc I dpp P' R' = (dpp1, dpp2)" by force
  note ss = s[symmetric]
  from 13(3) s have ok1: "isOK (check_dp_proof a ?i1 I dpp1 prof1)" 
    and ok2: "isOK (check_dp_proof a ?i2 I dpp2 prof2)" by auto
  from 13 have fin1: "\<forall>p\<in>set (dp_assms a prof1). finite_dppL p"
    and fin2: "\<forall>p\<in>set (dp_assms a prof2). finite_dppL p" by simp_all
  from 13(1)[OF ss HOL.refl ok1 13(4) fin1]
    have fin1': "finite_dpp (dpp_ops.dpp I dpp1)" by auto
  from 13(2)[OF ss HOL.refl _ ok2 13(4) fin2] ok1
    have fin2': "finite_dpp (dpp_ops.dpp I dpp2)" by auto
  show ?case
    by (rule split_proc[OF s fin1' fin2'])
next
  case (14 a i I dpp sli lP lQ lR prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 14(2) obtain dpp'
    where proc: "semlab_fin_proc I sli lP lQ lR dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 14 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case
    by (rule sound_proc_impl[OF semlab_fin_proc[OF 14(3)], OF proc 14(1)],
      unfold update_error_return, rule proc, rule rec, rule 14(3), rule fin)
next
  case (15 a i I dpp joins prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 15(2) obtain dpp'
    where proc: "switch_innermost_proc I joins dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 15 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case    
    by (rule sound_proc_impl[OF switch_innermost_proc[OF 15(3)] _ 15(1)], rule proc, unfold update_error_return, rule proc, rule rec, rule 15(3), rule fin)
next
  case (16 a i I dpp dpp')
  interpret dpp_spec I by fact
  have "a = True \<or> a = False" by simp
  thus ?case
  proof
    assume False: "a = False" thus ?thesis using 16(1) by simp
  next
    assume True: "a = True"
    show ?thesis
    proof (cases dpp')
      case fields
      show ?thesis using 16[unfolded True,simplified] and check_dpp_subsumes_sound by best
    qed
  qed
next
  case (17 a i I dpp U st st' lr p prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 17(2) obtain dpp'
    where proc: "rewriting_proc I U st st' lr p dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 17 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case    
    by (rule sound_proc_impl[OF rewriting_proc[OF 17(3)] _ 17(1)], rule proc, unfold update_error_return, rule proc, rule rec, rule 17(3), rule fin)  
next
  case (18 a i I dpp st p sts prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 18(2) obtain dpp'
    where proc: "narrowing_proc I st p sts dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 18 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case    
    by (rule sound_proc_impl[OF narrowing_proc[OF 18(3)] _ 18(1)], rule proc, unfold update_error_return, rule proc, rule rec, rule 18(3), rule fin)  
next
  case (19 a i I dpp st sts prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 19(2) obtain dpp'
    where proc: "instantiation_proc I st sts dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 19 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case    
    by (rule sound_proc_impl[OF instantiation_proc[OF 19(3)] _ 19(1)], rule proc, unfold update_error_return, rule proc, rule rec, rule 19(3), rule fin)  
next
  case (20 a i I dpp st sts u_opt prof)
  interpret dpp_spec I by fact
  let ?i = "i +@+ shows_string ''.1''"
  from 20(2) obtain dpp'
    where proc: "forward_instantiation_proc I st sts u_opt dpp = return dpp'"
    and rec: "isOK (check_dp_proof a ?i I dpp' prof)"
    by (auto simp: Let_def)
  from 20 have fin: "\<forall>p\<in>set (dp_assms a prof). finite_dppL p" by simp
  show ?case    
    by (rule sound_proc_impl[OF forward_instantiation_proc[OF 20(3)] _ 20(1)], rule proc, unfold update_error_return, rule proc, rule rec, rule 20(3), rule fin)  
qed

(*the old soundness lemma is an instance of the new one*)
lemma check_dp_proof:
  assumes "dpp_spec I"
    and "isOK (check_dp_proof False i I dpp prf)"
  shows "finite_dpp (dpp_ops.dpp I dpp)"
  using check_dp_proof_with_dp_assms[OF assms(1) _ assms(2)] by simp

end