(*
Copyright 2011 Christian Sternagel, René Thiemann

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory Nontermination_Impl
imports
  Nontermination
  "Q-Restricted_Rewriting_Impl"
  Labelings_Impl
  String_Reversal_Impl
begin

datatype ('f, 'v) dp_disproof =
  DP_Loop "('f, 'v) term" "('f, 'v) prseq" "('f, 'v) substL" "('f, 'v) ctxt"
| DP_Rule_Removal "('f, 'v) rules option" "('f, 'v) rules option" "('f, 'v) dp_disproof"

definition
  check_loop ::
    "('f,'v) term list \<Rightarrow> (('f::show, 'v::show) term \<Rightarrow> bool) \<Rightarrow>
     ('f, 'v) term \<Rightarrow>
     ('f, 'v) rseq \<Rightarrow> ('f, 'v) substL \<Rightarrow> ('f, 'v) ctxt \<Rightarrow>
     ('f, 'v) rules \<Rightarrow>
     shows check"
where
  "check_loop Q nf s rseq \<sigma> C R \<equiv> do {
    check (rseq \<noteq> []) (shows ''rewrite sequence must be non-empty'');
    if Q = []
      then check_qrsteps (\<lambda> _. True) R rseq s (C\<langle>s \<cdot> mk_subst \<sigma>\<rangle>)
      else do {
        check_qrsteps_subst (check_NF_iteration \<sigma> Q) R rseq s (C\<langle>s \<cdot> mk_subst \<sigma>\<rangle>)
      }
   }"

lemma check_loop_sound:
  assumes nf: "\<And>t. nf t \<longleftrightarrow> t \<in> NF_terms (set Q)"
    and ok: "isOK (check_loop Q nf s rseq \<sigma> C R)"
  shows "\<not> SN (qrstep (set Q) (set R))"
proof (cases "Q = []")
  case True
  obtain sig where sig: "sig = mk_subst \<sigma>" by auto
  with ok and True
    have check: "isOK (check_qrsteps (\<lambda> _. True) R rseq s (C\<langle>s \<cdot> sig\<rangle>))" and nonempty: "rseq \<noteq> []"
    by (auto simp: Let_def check_loop_def)
  from nonempty obtain n where rseq: "length rseq = Suc n" by (cases rseq, auto)
  with check_qrsteps_sound[OF _ check, of "[]"]
    have "(s,C\<langle>s \<cdot> sig\<rangle>) \<in> (rstep (set R)) ^^ (Suc n)" by auto
  hence step: "(s,C\<langle>s \<cdot> sig\<rangle>) \<in> (rstep (set R))^+" using pow_Suc_subset_trancl by auto
  from loop_imp_not_SN_on_rstep[OF step] show ?thesis
    unfolding SN_on_def True by auto
next
  case False
  let ?\<sigma> = "mk_subst \<sigma>"
  from False ok 
  have check: "isOK (check_qrsteps_subst (check_NF_iteration \<sigma> Q) R rseq s (C\<langle>s \<cdot> ?\<sigma>\<rangle>))" and nonempty: "rseq \<noteq> []"
    by (auto simp: Let_def check_loop_def)
  from nonempty obtain n where rseq: "length rseq = Suc n" by (cases rseq) simp_all
  from check_qrsteps_subst_sound[OF refl check]
  have "\<And> i. (s \<cdot> ?\<sigma> ^^ i, (C \<cdot>c ?\<sigma> ^^ i)\<langle>s \<cdot> ?\<sigma> \<cdot> ?\<sigma> ^^ i\<rangle>) \<in> qrstep (set Q) (set R) ^^ Suc n" unfolding rseq by auto
  hence step: "\<And> i. \<exists> C. (s \<cdot> ?\<sigma> ^^ i, C\<langle>s \<cdot> ?\<sigma> \<cdot> ?\<sigma> ^^ i\<rangle>) \<in> (qrstep (set Q) (set R))^+" using pow_Suc_subset_trancl by blast
  from loop_imp_not_SN_on_qrstep[OF step]
  show ?thesis unfolding SN_on_def by auto
qed

definition
  check_dp_loop ::
    "('f,'v)term list \<Rightarrow> (('f::show, 'v::show) term \<Rightarrow> bool) \<Rightarrow>
     ('f, 'v) term \<Rightarrow>
     ('f, 'v) prseq \<Rightarrow> ('f, 'v) substL \<Rightarrow> ('f, 'v) ctxt \<Rightarrow>
     ('f, 'v) rules \<Rightarrow> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_dp_loop Q nf s prseq \<sigma> C P R \<equiv> if C = \<box> then do {
    check (prseq \<noteq> []) (shows ''rewrite sequence must be non-empty'');
    if Q = [] then check_qsteps nf P R prseq s (s \<cdot> mk_subst \<sigma>)
        else check_qsteps_subst (check_NF_iteration \<sigma> Q) P R prseq s (s \<cdot> mk_subst \<sigma>)
  } else do {
    check_loop Q nf s (map (\<lambda>(x, y, _, z). (x, y, z)) prseq) \<sigma> C R
  }"
  
lemma check_dp_loop:
  assumes nf: "\<And>t. nf t \<longleftrightarrow> t \<in> NF_terms (set Q)"
    and ok: "isOK (check_dp_loop Q nf s prseq \<sigma> C P R)"
  shows "infinite_dpp (set P, set Q, set R)"
proof -
  obtain sig where sig: "sig = mk_subst \<sigma>" by auto
  show ?thesis
  proof (cases C)
    case Hole
    with ok  obtain n where prseq: "length prseq = Suc n" unfolding check_dp_loop_def
      by (cases prseq, auto)
    obtain PR where PR: "PR = rqrstep (set Q) (set P) \<union> qrstep (set Q) (set R)" by auto
    show ?thesis
    proof (cases "Q = []")
      case True
      with sig ok Hole
      have check: "isOK (check_qsteps nf P R prseq s (s \<cdot> sig))"
        by (auto simp: check_dp_loop_def)
      from check_qsteps_sound[OF nf check] prseq PR
      have "(s, s \<cdot> sig) \<in> PR ^^ (Suc n)" by simp
      hence step: "(s, s \<cdot> sig) \<in> PR^+" using pow_Suc_subset_trancl by auto
      from step[simplified PR True]
      have "(s, s \<cdot> sig) \<in> (rrstep (set P) \<union> rstep (set R))^+"
        unfolding rqrstep_def qrstep_rstep_conv rrstep_def' by simp
      from loop_imp_infinite_empty_Q[OF this]
      show ?thesis by (simp add: True)
    next
      case False
      with sig ok Hole
      have check: "isOK (check_qsteps_subst (check_NF_iteration \<sigma> Q) P R prseq s (s \<cdot> sig))"
        by (auto simp: check_dp_loop_def)
      from check_qsteps_subst_sound[OF refl check]
      have "\<And> i. (s \<cdot> sig ^^ i, s \<cdot> sig \<cdot> sig ^^ i) \<in> PR ^^ Suc n"
        unfolding sig prseq PR .
      hence step: "\<And> i. (s \<cdot> sig ^^ i, s \<cdot> sig \<cdot> sig ^^ i) \<in> PR^+" using pow_Suc_subset_trancl by blast
      from loop_imp_infinite[OF this[unfolded PR]]
      show ?thesis .
    qed      
  next
    case (More f bef D aft)
    with sig ok
      have check: "isOK (check_loop Q nf s (map (\<lambda> (x,y,_,z). (x,y,z)) prseq) \<sigma> C R)"
      by (auto simp: check_dp_loop_def)
    from check_loop_sound[OF nf this] and SN_subset
    have "\<not> SN (rqrstep (set Q) (set P) \<union> qrstep (set Q) (set R))"
      unfolding SN_on_def by blast
    thus ?thesis unfolding infinite_dpp_not_SN_conv .
  qed
qed

fun
  check_dp_disproof ::
    "shows \<Rightarrow> ('dpp, 'f::show, 'v::show) dpp_ops \<Rightarrow> 'dpp \<Rightarrow>
     ('f, 'v) dp_disproof \<Rightarrow>
     shows check"
where
  "check_dp_disproof i I dpp (DP_Loop s rseq \<sigma> C) = debug (i []) ''Loop'' (do {
    let P    = dpp_ops.pairs I dpp;
    let R    = dpp_ops.rules I dpp;
    let nf   = dpp_ops.is_QNF I dpp;
    let Q    = dpp_ops.Q I dpp;
    check_dp_loop Q nf s rseq \<sigma> C P R
      <+? (\<lambda> s. i +@+ '': Error in finding loop for the following DP-problem '' +#+
         shows_nl +@+ shows_dpp I dpp +@+ shows_nl +@+ indent s)
  })"
| "check_dp_disproof i I dpp (DP_Rule_Removal P' R' prf) = debug (i []) ''Rule Removal'' (do {
    let Prm = (if P' = None then [] else list_diff (dpp_ops.pairs I dpp) (the P'));
    let Rrm = (if R' = None then [] else list_diff (dpp_ops.rules I dpp) (the R'));
    check_dp_disproof (i +@+ shows_string ''.1'') I (
      dpp_spec.delete_pairs_rules I dpp Prm Rrm) prf
        <+? (\<lambda>s. i +@+ '': error below the pair/rule removal'' +#+ shows_nl
          +@+ indent s)
  })"

lemma check_dp_disproof:
  assumes I: "dpp_spec I"
    and ok: "isOK (check_dp_disproof i I dpp prf)"
  shows "infinite_dpp (set (dpp_ops.pairs I dpp), set (dpp_ops.Q I dpp), set (dpp_ops.rules I dpp))"
using assms
proof (induct "prf" arbitrary: dpp i)
  case (DP_Loop s rseq \<sigma> C dpp i)
  then interpret dpp_spec I by simp
  let ?Q    = "dpp_ops.Q I dpp"
  let ?nf   = "dpp_ops.is_QNF I dpp"
  let ?P    = "dpp_ops.pairs I dpp"
  let ?R    = "dpp_ops.rules I dpp"
  from DP_Loop have "isOK (check_dp_loop ?Q ?nf s rseq \<sigma> C ?P ?R)"
    unfolding check_dp_disproof.simps Let_def by simp
  with check_dp_loop[OF fun_cong[OF is_QNF_sound]]
    show ?case by blast
next
  case (DP_Rule_Removal P' R' "prf" dpp i)
  then interpret dpp_spec I by simp
  let ?P   = "dpp_ops.pairs I dpp"
  let ?Q   = "set (dpp_ops.Q I dpp)"
  let ?R   = "dpp_ops.rules I dpp"
  let ?Prm = "if P' = None then [] else list_diff ?P (the P')"
  let ?Rrm = "if R' = None then [] else list_diff ?R (the R')"
  let ?P'  = "set ?P - set ?Prm"
  let ?R'  = "set ?R - set ?Rrm"
  let ?dpp = "dpp_spec.delete_pairs_rules I dpp ?Prm ?Rrm"
  from DP_Rule_Removal
    have ok: "isOK (check_dp_disproof (i +@+ shows_string ''.1'') I ?dpp prf)"
    by (simp add: Let_def)
  from DP_Rule_Removal(1)[OF DP_Rule_Removal(2) ok]
    have "infinite_dpp (set (dpp_ops.pairs I ?dpp), set (dpp_ops.Q I ?dpp), set (dpp_ops.rules I ?dpp))"
      by simp
  hence inf: "infinite_dpp (?P', ?Q, ?R')" unfolding delete_simps .
  from inf
    have "(\<not> SN (qrstep ?Q ?R')) \<or> (\<exists>s t \<sigma>. Nontermination.ichain (?P', ?Q, ?R') s t \<sigma>)"
    (is "?notSN \<or> ?ichain")
    unfolding infinite_dpp.simps by blast
  thus ?case
  proof
    assume "?notSN"
    with SN_subset[OF _ qrstep_mono[of "?R'" "set ?R" "?Q" "?Q"]]
      have "\<not> SN (qrstep ?Q (set ?R))" by auto
    thus ?thesis by simp
  next
    assume ?ichain
    then obtain s t \<sigma> where "Nontermination.ichain (?P', ?Q, ?R') s t \<sigma>" by auto
    hence "Nontermination.ichain (set ?P, ?Q, ?R') s t \<sigma>" by auto
    with rtrancl_mono[OF qrstep_mono[of "?R'" "set ?R" "?Q" "?Q"]]
      have "Nontermination.ichain (set ?P, ?Q, set ?R) s t \<sigma>" by auto
    thus ?thesis by (simp only: infinite_dpp.simps) blast
  qed
qed

fun
  check_dps ::
    "('f \<Rightarrow> 'f) \<Rightarrow> ('f::show, 'v::show) rules \<Rightarrow> ('f, 'v) rules \<Rightarrow> ('f, 'v) rule check"
where
  "check_dps unshp r p = (
    let D = defs_list r in (
    check_all (\<lambda>(ll, rr). is_Fun rr \<and> (the (root rr)) \<notin> set D
      \<and> list_ex (\<lambda>(l, r). l = sharp_term unshp ll \<and> r \<unrhd> sharp_term unshp rr) r) p))"

lemma check_dps_sound:
  assumes check: "isOK (check_dps unshp r p)" 
    and infinite: "infinite_dpp (set p, set q, set r)"
  shows "\<not> SN (qrstep (set q) (set r))"
proof
  assume sn: "SN (qrstep (set q) (set r))"
  let ?ut = "sharp_term unshp"
  let ?rel = "supt \<union> qrstep (set q) (set r)"
  from infinite[unfolded infinite_dpp.simps] sn
    obtain s t \<sigma> where ichain: "Nontermination.ichain (set p, set q, set r) s t \<sigma>" by auto
  let ?t = "\<lambda>i. (?ut (t i) \<cdot> \<sigma> i)"
  have main: "\<And>i. (?t i, ?t (Suc i)) \<in> ?rel^+"
  proof -
    fix i    
    let ?sig = "\<sigma>"
    let ?i = "Suc i"
    from ichain have st: "(s i, t i) \<in> set p" by auto
    from ichain have st': "(s ?i, t ?i) \<in> set p" by auto
    from st check[unfolded check_dps.simps Let_def]
      obtain g ts where ti: "t i = Fun g ts"
      and ndef: "\<not> defined (set r) (g, length ts)"
      unfolding defs_list_sound by (cases "t i") (force+)
    from st' check[unfolded check_dps.simps]
      obtain l rr where lr: "(l,rr) \<in> set r" and si': "l = ?ut (s ?i)"
      and rr: "rr \<unrhd> ?ut (t ?i)" by (auto simp: list_ex_iff)
    from left_Var_imp_not_SN_qrstep[of _ rr "set r" "set q"] sn lr
      obtain ff ss where l: "l = Fun ff ss" by (cases l, force)
    from si' l obtain f where si': "s ?i = Fun f ss"
      and f: "ff = unshp f" by (cases "s ?i", auto)
    from si' f l have l: "l = Fun (unshp f) ss" by simp
    have steps: "(?ut (t i \<cdot> \<sigma> i), ?ut (s (Suc i) \<cdot> \<sigma> (Suc i))) \<in> (qrstep (set q) (set r))^*"
    proof -
      from ichain have steps: "(t i \<cdot> \<sigma> i, s (Suc i) \<cdot> \<sigma> (Suc i)) \<in> (qrstep (set q) (set r))^*" by simp
      have "(t i \<cdot> \<sigma> i, s (Suc i) \<cdot> \<sigma> (Suc i)) \<in> (nrqrstep (set q) (set r))^*"
      proof (rule qrsteps_imp_nrqrsteps[OF _ _ steps])
        show "\<not> defined (applicable_rules (set q) (set r)) (the (root (t i \<cdot> \<sigma> i)))"
          unfolding ti using ndef_applicable_rules[OF ndef, of "set q"] by auto
      next
        {
          fix l rr
          assume "(l,rr) \<in> set r" and var: "is_Var l"
          with SN_imp_wwf_qtrs[OF sn, unfolded wwf_qtrs_def]
          have "\<not> applicable_rule (set q) (l,rr)" by auto
          with left_Var_applicable[of "set q" _ rr] var have False
            by (cases l, auto)
        }
        thus "\<forall> (l,rr) \<in> set r. is_Fun l" by auto
      qed
      from nrqrsteps_imp_sharp_qrsteps[OF this]
        show ?thesis .
    qed
    from rtrancl_mono[of "qrstep (set q) (set r)" "supt \<union> qrstep (set q) (set r)"]
      have "(qrstep (set q) (set r))^* \<subseteq> ?rel^*" by auto
    from set_mp[OF this steps]
      have steps: "(?ut (t i) \<cdot> ?sig i, ?ut (s ?i) \<cdot> ?sig ?i) \<in> ?rel^*"
      unfolding si' ti by auto
    from ichain have "\<forall> u \<lhd> s ?i \<cdot> ?sig ?i. u \<in> NF_terms (set q)" by auto
    hence NF: "\<forall>u\<lhd> l \<cdot> \<sigma> (Suc i). u \<in> NF_terms (set q)" unfolding l si'
      unfolding NF_terms_args_conv[symmetric] by auto      
    have "(?ut (s ?i) \<cdot> ?sig ?i, rr \<cdot> ?sig ?i) \<in> ?rel"
      using qrstepI[OF NF lr refl refl, of \<box>] unfolding si' l by auto
    moreover from supteq_stable[OF rr, of "?sig ?i"]
      have "(rr \<cdot> ?sig ?i, ?ut (t ?i) \<cdot> ?sig ?i) \<in> ?rel^=" 
      by (auto simp: supteq_def)
    ultimately have "(?ut (s ?i) \<cdot> ?sig ?i, ?ut (t ?i) \<cdot> ?sig ?i) \<in> ?rel O ?rel^=" by auto
    with steps
    have "(?ut (t i) \<cdot> ?sig i, ?ut (t (Suc i)) \<cdot> ?sig (Suc i)) \<in> ?rel^* O ?rel O ?rel^="
      (is "?pair \<in> _") by blast
    hence "?pair \<in> ?rel^+ O ?rel^=" by (simp only:  O_assoc[symmetric] trancl_unfold_right)
    thus "?pair \<in> ?rel^+" by (simp only: trancl_o_refl_is_trancl)
  qed
  hence "\<not> SN_on (?rel^+) {?t 0}" unfolding SN_on_def by auto
  hence "\<not> SN_on ?rel {?t 0}" using SN_on_trancl[of ?rel] by auto
  hence "\<not> SN_on (qrstep (set q) (set r)) {?t 0}"
    using SN_on_qrstep_imp_SN_on_supt_union_qrstep by auto
  with sn show False unfolding SN_on_def by auto
qed

datatype ('f, 'v) "trs_disproof" = 
  TRS_Loop "('f, 'v) term" "('f, 'v) rseq" "('f, 'v) substL" "('f, 'v) ctxt"
| TRS_Not_Well_Formed
| TRS_Rule_Removal "('f, 'v) rules" "('f, 'v) trs_disproof"
| TRS_String_Reversal "('f, 'v) trs_disproof"
| TRS_DP_Trans "('f, 'v) rules" "('f, 'v) dp_disproof"

fun unsharp :: "('f, 'l) lab \<Rightarrow> ('f, 'l) lab" where
  "unsharp (Sharp f) = f"
| "unsharp f = f"

definition
  applicable_rule_impl :: "(('f, 'v) term \<Rightarrow> bool) \<Rightarrow> ('f, 'v) rule \<Rightarrow> bool"
where
  "applicable_rule_impl nf r \<equiv> list_all nf (args (fst r))"

lemma applicable_rule_impl_sound[simp]:
  assumes nf: "\<And>t. nf t \<longleftrightarrow> t \<in> NF_terms (set Q)"
  shows "applicable_rule_impl nf r \<longleftrightarrow> applicable_rule (set Q) r"
  unfolding applicable_rule_impl_def applicable_rule_def list_all_iff NF_terms_args_conv[symmetric] unfolding nf[symmetric]
   by simp

definition
  check_wwf_qtrs :: "(('f::show, 'v::show) term \<Rightarrow> bool) \<Rightarrow> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_wwf_qtrs nf R \<equiv> (
    check_allm (\<lambda>r. if applicable_rule_impl nf r
      then (do {
        check (is_Fun (fst r)) (shows_string ''variable left-hand side in'');
        check_subseteq (vars_term_list (snd r)) (vars_term_list (fst r))
          <+? (\<lambda>x. ''free variable '' +#+ shows x
            +@+ shows_string '' in right-hand side of'')
      } <+? (\<lambda>s. s +@+ '' rule '' +#+ shows_rule r +@+ shows_nl)) else return ()) R
      <+? (\<lambda>e. ''the Q-TRS is not weakly well-formed'' +#+ shows_nl +@+ e))"

lemma check_wwf_qtrs_sound[simp]:
  assumes nf: "\<And>t. nf t \<longleftrightarrow> t \<in> NF_terms (set Q)"
  shows "isOK (check_wwf_qtrs nf R) = wwf_qtrs (set Q) (set R)"
  unfolding check_wwf_qtrs_def wwf_qtrs_def applicable_rule_impl_sound[OF nf]
    split_def by simp

definition
  check_not_wwf_qtrs :: "(('f::show, 'v::show) term \<Rightarrow> bool) \<Rightarrow> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_not_wwf_qtrs nf R \<equiv>
     check (\<not> isOK (check_wwf_qtrs nf R)) (''The Q-TRS is well formed'' +#+ shows_nl)"

lemma check_not_wwf_qtrs_sound:
  assumes nf: "\<And>t. nf t \<longleftrightarrow> t \<in> NF_terms (set Q)"
  assumes "isOK (check_not_wwf_qtrs nf R)"
  shows "\<not> SN (qrstep (set Q) (set R))"
proof -
  from assms have "\<not> wwf_qtrs (set Q) (set R)" unfolding check_not_wwf_qtrs_def by auto
  with rev_contrapos[OF SN_imp_wwf_qtrs] show ?thesis by auto
qed

fun
  check_trs_disproof ::
    "shows \<Rightarrow>
     ('tp, ('f::show, 'l::show) lab, 'v::show) tp_ops \<Rightarrow>
     ('dpp, ('f, 'l) lab, 'v) dpp_ops \<Rightarrow>
     'tp \<Rightarrow>
     (('f, 'l) lab, 'v) trs_disproof \<Rightarrow>
     shows check"
where
  "check_trs_disproof i I J tp (TRS_Loop s rseq \<sigma> C) = debug (i []) ''Loop'' (do {
    let R    = tp_ops.rules I tp;
    let nf   = tp_ops.is_QNF I tp;
    let Q    = tp_ops.Q I tp;
    check_loop Q nf s rseq \<sigma> C R
  })"
| "check_trs_disproof i I J tp TRS_Not_Well_Formed = debug (i []) ''Not Well-Formed'' (
    check_not_wwf_qtrs (tp_ops.is_QNF I tp) (tp_ops.rules I tp)
      <+? (\<lambda>s. i +@+ '': error in not well-formed proof'' +#+ shows_nl +@+ s))"
| "check_trs_disproof i I J tp (TRS_Rule_Removal R' prf) = debug (i []) ''Rule Removal'' (do {
    let R   = tp_ops.rules I tp;
    let Rrm = list_diff R R';
    check_trs_disproof (i +@+ shows_string ''.1'') I J (
      tp_spec.delete_rules I tp Rrm) prf
        <+? (\<lambda>s. i +@+ '': error below the rule removal'' +#+ shows_nl
          +@+ indent s)
  })"
| "check_trs_disproof i I J tp (TRS_String_Reversal prf) =
    debug (i []) ''String Reversal'' (do {
      let R = tp_ops.rules I tp;
      check (tp_ops.Q_empty I tp) (shows ''Q is not empty'');
      tp' \<leftarrow> do {
        check_return (check_unary_signature R) (tp_ops.mk I [] (map rev_rule R) [])
      } <+? (\<lambda>s. i +@+ '': error when applying string reversal on'' +#+ shows_nl
        +@+ shows_tp I tp +@+ shows_nl +@+ s);
      check_trs_disproof (i +@+ shows_string ''.1'') I J tp' prf
        <+? (\<lambda>s. i +@+ '': error below the string reversal technique'' +#+ shows_nl
          +@+ indent s)
    })"
| "check_trs_disproof i I J tp (TRS_DP_Trans P prf) = debug (i []) ''DP Transformation'' (do {
     let R = tp_ops.rules I tp;
     let Q = tp_ops.Q I tp;
     check_dps unsharp R P
       <+? (\<lambda>lr. i +@+ '': error in DP transformation'' +#+ shows_nl
         +@+ shows ''problematic rule '' +@+ shows_rule lr);
     let dpp = dpp_ops.mk J P [] Q [] R;
     check_dp_disproof (i +@+ shows_string ''.1'') J dpp prf
        <+? (\<lambda>s. i +@+ '': error below the dp transformation'' +#+ shows_nl
          +@+ indent s)
  })"

lemma check_trs_disproof_TRS_Rule_RemovalD[dest]:
  assumes "isOK (check_trs_disproof i I J tp (TRS_Rule_Removal R prf))"
  shows "isOK (check_trs_disproof (i +@+ shows_string ''.1'') I J
    (tp_spec.delete_rules I tp (list_diff (tp_ops.rules I tp) R)) prf)"
  using assms by (simp add: Let_def)

lemma check_trs_disproof_TRS_String_ReversalD[dest]:
  assumes "isOK (check_trs_disproof i I J tp (TRS_String_Reversal prf))"
  shows "isOK (check_trs_disproof (i +@+ shows_string ''.1'') I J
    (tp_ops.mk I [] (map rev_rule (tp_ops.rules I tp)) []) prf)"
  using assms by (auto simp: Let_def)

lemma check_trs_disproof_TRS_DP_TransD[dest]:
  assumes "isOK (check_trs_disproof i I J tp (TRS_DP_Trans P prf))"
  shows "isOK (check_dp_disproof (i +@+ shows_string ''.1'') J
    (dpp_ops.mk J P [] (tp_ops.Q I tp) [] (tp_ops.rules I tp)) prf)"
  using assms by (simp add: Let_def)

lemma check_trs_disproof_sound:
  assumes I: "tp_spec I" and J: "dpp_spec J"
    and ok: "isOK (check_trs_disproof i I J tp prf)"
  shows "\<not> SN (qrstep (set (tp_ops.Q I tp)) (set (tp_ops.rules I tp)))"
    (is "\<not> SN (qrstep ?Q ?R)")
using ok
proof (induct "prf" arbitrary: tp i)
  interpret tp_spec I by (simp add: I)
  case (TRS_Not_Well_Formed tp i)
  hence "isOK (check_not_wwf_qtrs (is_QNF tp) (rules tp))" by simp
  from check_not_wwf_qtrs_sound[OF _ this] show ?case by simp
next
  interpret tp_spec I by (simp add: I)
  case (TRS_Loop s rseq \<sigma> C tp i)
  let ?R    = "tp_ops.rules I tp"
  let ?nf   = "tp_ops.is_QNF I tp"
  let ?Q = "tp_ops.Q I tp"
  from TRS_Loop have "isOK (check_loop ?Q ?nf s rseq \<sigma> C ?R)" by (simp add: Let_def)
  from check_loop_sound[OF _ this] show ?case by simp
next
  interpret tp_spec I by (simp add: I)
  case (TRS_Rule_Removal R' "prf" tp i)
  let ?R   = "(rules tp)"
  let ?Q   = "set (Q tp)"
  let ?Rrm = "list_diff ?R R'"
  let ?R'  = "set ?R - set ?Rrm"
  let ?tp' = "delete_rules tp ?Rrm"
  from TRS_Rule_Removal
    have "isOK (check_trs_disproof (i +@+ shows_string ''.1'') I J ?tp' prf)"
    by (simp add: Let_def)
  from TRS_Rule_Removal(1)[OF this]
    have "\<not> SN (qrstep (set (Q ?tp')) (set (rules ?tp')))" by simp
  hence "\<not> SN (qrstep ?Q ?R')"
    unfolding delete_simps .
  with SN_subset[OF _ qrstep_mono[of _ _ "?Q" "?Q"]]
    show ?case by blast
next
  interpret tp_spec I by (simp add: I)
  case (TRS_String_Reversal "prf" tp i)
  note IH = this
  let ?R = "set (tp_ops.rules I tp)"
  let ?Q = "set (tp_ops.Q I tp)"
  let ?tp' = "tp_ops.mk I [] (map rev_rule (tp_ops.rules I tp)) []"
  from IH have "tp_ops.Q_empty I tp" by (simp add: Let_def)
  hence Q: "?Q = {}" by simp
  from IH(2) have sig: "unary_sig (funas_trs ?R)" by (simp add: Let_def check_return_def)
  from IH(2) have rev: "set (tp_ops.rules I ?tp') = rev_trs ?R"
    by (auto simp: Let_def check_return_def rev_trs_def)
  from IH have "\<not> SN (qrstep (set (tp_ops.Q I ?tp')) (set (tp_ops.rules I ?tp')))" by blast
  hence "\<not> SN (qrstep {} (rev_trs ?R))" unfolding rev by simp
  with string_reversal[OF sig]
    show ?case unfolding Q qrstep_rstep_conv by simp
next
  interpret tp!: tp_spec I by (simp add: I)
  interpret dpp!: dpp_spec J by (simp add: J)
  case (TRS_DP_Trans P "prf" tp i)
  note IH = this
  let ?R = "tp_ops.rules I tp"
  let ?Q = "tp_ops.Q I tp"
  let ?dpp = "dpp_ops.mk J P [] ?Q [] ?R"
  from IH have ok_dps: "isOK (check_dps unsharp ?R P)" by (simp add: Let_def)
  from IH have ok: "isOK (check_dp_disproof (i +@+ shows_string ''.1'') J ?dpp prf)" by blast
  from check_dp_disproof[OF `dpp_spec J` ok]
  have "infinite_dpp (set (dpp_ops.pairs J ?dpp), set (dpp_ops.Q J ?dpp), set (dpp_ops.rules J ?dpp))"
    .
  hence "infinite_dpp (set P, set ?Q, set ?R)"
    by (simp add: dpp.mk_simps dpp.pairs_sound dpp.rules_sound)
  thus ?case using check_dps_sound[OF ok_dps]
    unfolding qrstep_rstep_conv by blast
qed

fun
  check_rel_seq ::
    "('f::show, 'v::show) rules \<Rightarrow> ('f, 'v) rules \<Rightarrow>
     ('f, 'v) prseq \<Rightarrow> ('f, 'v) term \<Rightarrow> ('f, 'v) term \<Rightarrow> bool \<Rightarrow>
     shows check"
where
  "check_rel_seq R S [] s u True = check (s = u) (
     ''the last term of the rewrite sequence'' +#+ shows_nl +@+ shows s +@+ shows_nl
     ''does not correspond to the goal term''  +#+ shows_nl +@+ shows u +@+ shows_nl
   )"
| "check_rel_seq R S [] s u False = error (shows ''did not find strict step in rewrite sequence'')"
| "check_rel_seq R S ((p, r, True, t) # prts) s u b = do {
     check_qrstep (\<lambda> _ . True) R p r s t;
     check_rel_seq R S prts t u True
   }"
| "check_rel_seq R S ((p, r, False, t) # prts) s u b = do {
     check_qrstep (\<lambda> _ . True) S p r s t;
     check_rel_seq R S prts t u b
   }"

lemma check_rel_seq_rtrancl:
  assumes "isOK (check_rel_seq R S rseq s t b)"
  shows "(s, t) \<in> (rstep (set R \<union> set S))^*" (is "_ \<in> ?R^*")
using assms
proof (induct rseq arbitrary: s b)
  case Nil
  thus ?case by (cases b) simp_all
next
  case (Cons prt rseq)
  obtain p r or u where prt: "prt = (p, r, or, u)" by (cases prt) blast
  from Cons[unfolded prt]
  have IH: "(u, t) \<in> ?R^*" by (cases or, induct rseq) auto
  have "(s,u) \<in> ?R"
  proof (cases or)
    case True
    with Cons[unfolded prt]
    have "isOK (check_qrstep (\<lambda> _ . True) R p r s u)" by simp
    from check_qrstep_qrstep[OF _ this, of "[]"]
    have "(s,u) \<in> rstep (set R)" by auto
    thus ?thesis by auto
  next
    case False
    with Cons[unfolded prt]
    have "isOK (check_qrstep (\<lambda> _ . True) S p r s u)" by simp
    from check_qrstep_qrstep[OF _ this, of "[]"]
    have "(s,u) \<in> rstep (set S)" by simp
    thus ?thesis by auto
  qed
  with IH show ?case by auto
qed

lemma check_rel_seq_trancl:
  assumes "isOK (check_rel_seq R S rseq s t False)"
  shows "(s, t) \<in> (rstep (set R \<union> set S))^* O rstep (set R) O (rstep (set R \<union> set S))^*"
proof -
  let ?RS = "(rstep (set R \<union> set S))^*"
  let ?R  = "?RS O rstep (set R) O ?RS"
  from assms show "(s,t) \<in> ?R"
  proof (induct rseq arbitrary: s)
    case (Cons prt rseq)
    obtain p r or u where prt: "prt = (p, r, or, u)" by (cases prt) blast   
    show ?case 
    proof (cases or)
      case False
      with Cons[unfolded prt]
      have IH: "(u, t) \<in> ?R"  and check: "isOK (check_qrstep (\<lambda> _ . True) S p r s u)" by auto
      have subset: "rstep (set R \<union> set S) O ?RS \<subseteq> ?RS" by auto
      from check_qrstep_qrstep[OF _ check, of "[]"] have "(s, u) \<in> rstep (set R \<union> set S)" by auto
      with IH have "(s, t) \<in> (rstep (set R \<union> set S) O ?RS) O rstep (set R) O ?RS"
        by (auto simp: O_assoc)
      thus ?thesis using subset by blast
    next
      case True
      with Cons[unfolded prt]
        have check1: "isOK (check_qrstep (\<lambda> _ . True) R p r s u)"
        and check2: "isOK (check_rel_seq R S rseq u t True)"
        by auto
      from check_qrstep_qrstep[OF _ check1, of "[]"]  check_rel_seq_rtrancl[OF check2]
        show ?thesis by auto
    qed
  qed simp
qed

definition
  check_rel_loop ::
    "('f::show, 'v::show) term \<Rightarrow>
     ('f, 'v) prseq \<Rightarrow> ('f, 'v) substL \<Rightarrow> ('f, 'v) ctxt \<Rightarrow>
     ('f, 'v) rules \<Rightarrow> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_rel_loop s rseq \<sigma> C R S \<equiv> check_rel_seq R S rseq s (C\<langle>s \<cdot> mk_subst \<sigma>\<rangle>) False"

lemma check_rel_loop:
  assumes "isOK (check_rel_loop s rseq \<sigma> C R S)"
  shows "\<not> (SN_qrel ({}, (set R), (set S)))"
  by (rule loop_imp_not_SN_qrel_empty_Q[OF
    check_rel_seq_trancl[OF assms[unfolded check_rel_loop_def]]])

fun
  check_wf_reltrs :: "('f::show, 'v::show) rules \<times> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_wf_reltrs (R, S) = (do {
     check_wf_trs R;
     (if R = [] then OK else
     check_allm (\<lambda>rule. do {
       check_subseteq (vars_term_list (snd rule)) (vars_term_list (fst rule))
         <+? (\<lambda>x. ''free variable '' +#+ shows x
               +@+ shows_string '' in right-hand side of'')
     } <+? (\<lambda>s. s +@+ '' rule '' +#+ shows_rule rule +@+ shows_nl)
     ) S)
   }
   )"

lemma check_wf_reltrs[simp]:
  "isOK (check_wf_reltrs (R, S)) = wf_reltrs (set R, set S)"
proof
  assume check: "isOK (check_wf_reltrs (R,S))"
  hence wf_trs: "wf_trs (set R)" by auto
  show "wf_reltrs (set R, set S)"
  proof (cases R)
    case Nil
    with wf_trs show ?thesis by simp
  next
    case (Cons lr R')
    with check have "\<forall>(l, r)\<in>set S. vars_term r \<subseteq> vars_term l" by force
    with wf_trs Cons show ?thesis by auto
  qed
next
  assume wf_reltrs: "wf_reltrs (set R, set S)"
  thus "isOK (check_wf_reltrs (R,S))"
    by (cases R, auto)
qed

declare check_wf_reltrs.simps[simp del]

definition
  check_not_wf_reltrs :: "('f::show, 'v::show) rules \<times> ('f, 'v) rules \<Rightarrow> shows check"
where
  "check_not_wf_reltrs RS \<equiv>
     check (\<not> isOK (check_wf_reltrs RS)) (''The TRSs R and S are well formed'' +#+ shows_nl)"

lemma check_not_wf_reltrs:
  assumes check: "isOK (check_not_wf_reltrs (R,S))"
  shows "\<not> rel_SN (rstep (set R), rstep (set S))"
proof -
  from assms have "\<not> wf_reltrs (set R, set S)" unfolding check_not_wf_reltrs_def by auto
  with rel_SN_imp_wf_reltrs show ?thesis by blast
qed

datatype ('f, 'v)"reltrs_disproof" = 
  Rel_Loop "('f, 'v) term" "('f, 'v) prseq" "('f, 'v) substL" "('f, 'v) ctxt"
| Rel_Not_Well_Formed
| Rel_Rule_Removal "('f, 'v) rules option" "('f, 'v) rules option" "('f, 'v) reltrs_disproof"
| Rel_R_Not_SN "('f, 'v) trs_disproof"

fun
  check_reltrs_disproof ::
    "shows \<Rightarrow>
     ('tp, ('f::show, 'l::show) lab, 'v::show) tp_ops \<Rightarrow>
     ('dpp, ('f, 'l) lab, 'v) dpp_ops \<Rightarrow>
     'tp \<Rightarrow>
     (('f, 'l) lab, 'v) reltrs_disproof \<Rightarrow>
     shows check"
where
  "check_reltrs_disproof i I J tp (Rel_Loop s rseq \<sigma> C) = debug (i []) ''Loop'' (do {
    let R = tp_ops.R I tp;
    let S = tp_ops.Rw I tp;
    check (tp_ops.Q_empty I tp) (shows ''Q is not empty'');
    check_rel_loop s rseq \<sigma> C R S
      <+? (\<lambda>s. i +@+ '': error in loop proof'' +#+ shows_nl +@+ s)
  })"
| "check_reltrs_disproof i I J tp (Rel_R_Not_SN prf) = do {
    let Q = tp_ops.Q I tp;
    let R = tp_ops.R I tp;
    let tp' = tp_ops.mk I Q R [];
    check_trs_disproof (i +@+ shows_string ''.1'') I J tp' prf
        <+? (\<lambda>s. i +@+ '': error below the nontermination proof for R'' +#+ shows_nl
          +@+ indent s)
  }"
| "check_reltrs_disproof i I J tp Rel_Not_Well_Formed = debug (i []) ''Not Well-Formed'' (do {
    let R = tp_ops.R I tp;
    let S = tp_ops.Rw I tp;
    check (tp_ops.Q_empty I tp) (shows ''Q is not empty'');
    check_not_wf_reltrs (R, S)
      <+? (\<lambda>s. i +@+ '': error in not-well-formed proof'' +#+ shows_nl +@+ s)
  })"
| "check_reltrs_disproof i I J tp (Rel_Rule_Removal R' S' prf) =
    debug (i []) ''Rule Removal'' (do {
      let Rrm = (if R' = None then [] else list_diff (tp_ops.R I tp) (the R'));
      let Srm = (if S' = None then [] else list_diff (tp_ops.Rw I tp) (the S'));
      check_reltrs_disproof (i +@+ shows_string ''.1'') I J (
        tp_ops.delete_R_Rw I tp Rrm Srm) prf
        <+? (\<lambda>s. i +@+ '': error below the rule removal'' +#+ shows_nl
          +@+ indent s)
    })"

lemma SN_qrel_imp_SN_qrstep:
  assumes "SN_qrel (Q, R, S)"
  shows "SN (qrstep Q R)"
  using assms unfolding SN_qrel_def split_def fst_conv snd_conv
  using rel_SN_imp_SN[of "qrstep Q R"] by blast

lemma check_reltrs_disproof_sound:
  assumes I: "tp_spec I" and J: "dpp_spec J"
    and ok: "isOK (check_reltrs_disproof i I J tp prf)"
  shows "\<not> SN_qrel (set (tp_ops.Q I tp), set (tp_ops.R I tp), set (tp_ops.Rw I tp))"
    (is "\<not> SN_qrel (?Q, ?R, ?S)")
using ok
proof (induct "prf" arbitrary: tp i)
  interpret tp_spec I by (simp add: I)
  case (Rel_Loop s rseq \<sigma> C tp i)
  note IH = this
  let ?Q = "tp_ops.Q I tp"
  let ?R = "tp_ops.R I tp"
  let ?S = "tp_ops.Rw I tp"
  from IH have Q: "set ?Q = {}" by (simp add: Q_empty_sound)
  from IH have "isOK (check_rel_loop s rseq \<sigma> C ?R ?S)" by simp
  from check_rel_loop[OF this] show ?case by (simp add: Q)
next
  interpret tp_spec I by (simp add: I)
  case Rel_Not_Well_Formed
  note IH = this
  let ?Q = "tp_ops.Q I tp"
  let ?R = "tp_ops.R I tp"
  let ?S = "tp_ops.Rw I tp"
  from IH have Q: "set ?Q = {}" by (simp add: Q_empty_sound)
  from IH have "isOK (check_not_wf_reltrs (?R, ?S))" by simp
  from check_not_wf_reltrs[OF this]
    show ?case by (simp add: Q)
next
  interpret tp_spec I by (simp add: I)
  case (Rel_Rule_Removal R' S' "prf")
  note IH = this
  let ?Q   = "set (tp_ops.Q I tp)"
  let ?R   = "tp_ops.R I tp"
  let ?Rrm = "if R' = None then [] else list_diff ?R (the R')"
  let ?S   = "tp_ops.Rw I tp"
  let ?Srm = "if S' = None then [] else list_diff ?S (the S')"
  let ?R'  = "set ?R - set ?Rrm"
  let ?S'  = "set ?S - set ?Srm"
  let ?tp' = "tp_ops.delete_R_Rw I tp ?Rrm ?Srm"
  from IH
    have "\<not> SN_qrel (set (tp_ops.Q I ?tp'), set (tp_ops.R I ?tp'), set (tp_ops.Rw I ?tp'))"
    by force
  hence "\<not> SN_qrel (?Q, ?R', ?S')" unfolding delete_simps .
  with SN_qrel_mono[of "?Q" "?Q" "?R'" "set ?R" "?S'" "set ?S"]
    show ?case by blast
next
  interpret tp_spec I by (simp add: I)
  case (Rel_R_Not_SN "prf")
  note IH = this
  let ?Q = "tp_ops.Q I tp"
  let ?R = "tp_ops.R I tp"
  let ?tp' = "tp_ops.mk I ?Q ?R []"
  from IH have "isOK (check_trs_disproof (i +@+ shows_string ''.1'') I J ?tp' prf)" by simp
  from check_trs_disproof_sound[OF I J this]
    have "\<not> SN (qrstep (set (tp_ops.Q I ?tp')) (set (tp_ops.rules I ?tp')))" by simp
  hence "\<not> SN (qrstep (set ?Q) (set ?R))" by (simp add: mk_simps)
  with SN_qrel_imp_SN_qrstep show ?case by blast
qed

end
